-- kEnergy
-- Simple energy ticker addon inspired by GonffBar

local name = 'kEnergy'
local ticktime = 2
local fontsize = 12

local function keys(t, k)
    local nk = next(t, k)
    if nk == nil then
        return
    end
    return nk, keys(t, nk)
end
local function empty() return '' end


local candy = AceLibrary('CandyBar-2.0')
local paint = AceLibrary('PaintChips-2.0')
local L = AceLibrary('AceLocale-2.2'):new(name)
local addon = AceLibrary('AceAddon-2.0'):new('AceDB-2.0', 'AceEvent-2.0', 'AceConsole-2.0', 'CandyBar-2.0')

function addon:OnInitialize()
    self.surface = AceLibrary('Surface-1.0')
    self.surface:Register('BantoBar', 'Interface\\AddOns\\kEnergy\\Textures\\BantoBar')

    self.options = {
        type = 'group',
        args = {
            width = {
                name  = L['Width'],
                desc  = L['Set the width of the bar.'],
                type  = 'range',
                min   = 25,
                max   = 400,
                get   = function() return self.db.profile.width end,
                set   = function(w)
                    self.db.profile.width = w
                    self.frame:SetWidth(w)
                    self:SetCandyBarWidth(name, w)

                    if not self.locked then
                        candy:Update(name)
                    end
                end
            },
            height = {
                type  = 'range',
                name  = L['Height'],
                desc  = L['Set the height of the bar.'],
                min   = 5,
                max   = 100,
                get   = function() return self.db.profile.height end,
                set   = function(h)
                    self.db.profile.height = h
                    self.frame:SetHeight(h)
                    self:SetCandyBarHeight(name, h)

                    if not self.locked then
                        candy:Update(name)
                    end
                end
            },
            texture = {
                type = 'text',
                name = L['Texture'],
                desc = L['Set the texture of the bar.'],
                get  = function() return self.db.profile.texture end,
                set  = function(v)
                    self.db.profile.texture = v
                    self:SetCandyBarTexture(name, self.surface:Fetch(v))
                end,
                validate = self.surface:List()
            },
            color = {
                type = 'text',
                name = L['Color'],
                desc = L['Set the color of the bar.'],
                get  = function() return self.db.profile.color end,
                set  = function(v)
                    self.db.profile.color = v
                    self:SetCandyBarColor(name, v)
                end,
                validate = {keys(paint.vars.hex)},
            },
            lock = {
                type = 'toggle',
                name = L['Lock'],
                desc = L['Lock the bar.'],
                get  = function() return self.locked end,
                set  = function()
                    self.locked = not self.locked

                    if self.locked then
                        self:StopCandyBar(name)
                        self:SetCandyBarText(name, '')
                    else
                        self:StartCandyBar(name)
                        self:SetCandyBarTimeLeft(name, ticktime)
                        self:SetCandyBarText(name, name)
                        self:PauseCandyBar(name)
                        candy:Update(name)
                    end
                end,
            },
        }
    }
    self.locked = true

    self:RegisterChatCommand({'/kenergy', '/ke'}, self.options)
    self:RegisterDB('kEnergySaved')
    self:RegisterDefaults('profile', {
        width   = 225,
        height  = 25,
        texture = 'BantoBar',
        color   = 'rogue',
    })
end

function addon:OnEnable()
    local _, class = UnitClass('player')
    class = class:upper()

    if (class ~= 'DRUID') and (class ~= 'ROGUE') then
        DisableAddOn(name)
        return
    end

    self:RegisterEvent('UNIT_ENERGY')
    self:RegisterEvent('Surface_Registered', function()
        self.options.args.texture.validate = self.surface:List()
    end)

    self:RegisterCandyBar(name, ticktime, '', nil, self.db.profile.color)
    self:SetCandyBarHeight(name, self.db.profile.height)
    self:SetCandyBarWidth(name, self.db.profile.width)
    self:SetCandyBarFade(name, 1)
    self:SetCandyBarTexture(name, self.surface:Fetch(self.db.profile.texture))
    self:SetCandyBarTimeFormat(name, empty)
    self:SetCandyBarFontSize(name, fontsize)

    self:CreateFrame()
    self:RestorePosition()
end

function addon:UNIT_ENERGY()
    if self.locked and UnitMana('player') ~= UnitManaMax('player') then
        self:StartCandyBar(name)
    end
end


_G[name] = addon
